local ADDON_NAME="OrderOfBattle"
local _G = _G
local pairs = pairs
local ipairs = ipairs
local remove = table.remove
local type = type
local select = select
local next = next

local Core=OrderOfBattle
local PlayerFrame=OOBPlayerFrame

local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME, true)

AssignmentFrame = {Embed = Core.Embed}
AssignmentEditor = {Embed = Core.Embed}
AssignmentFrame.frameCounter=0

function AssignmentFrame:New(parent)
	if self ~= AssignmentFrame then return end
	local s=CreateFrame("FRAME",nil,parent,nil)
	AssignmentFrame:Embed(s)	
	
	s.frameID=AssignmentFrame.frameCounter
	AssignmentFrame.frameCounter=AssignmentFrame.frameCounter+1

	s:CreateBaseFrame()
	
	s.editor=AssignmentEditor:New(s)

	s.prevFrame=nil
	s.uiElements={}
	s.dropTargets=nil
	s:UpdatePoints()
	
	s:SetBackdrop({bgFile = nil, 
					edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
					tile = true, tileSize = 16, edgeSize = 16, 
					insets = { left = 0, right = 0, top = 0, bottom = 0 }})
						
	s:Show()
	
	return s
end

function AssignmentFrame:Dispose()
	local i,e,v,pf
	for i,e in ipairs(self:GetDropTargets()) do
		if e.type==PlayerFrame.TYPE_DROP_FULL then
			e:DropTo(nil)
		end
	end	
	self:ClearAssignmentUI()
end

function AssignmentFrame:FindMouseTarget()
	local i,f,d
	local mx,my=GetCursorPosition()
	mx=mx/UIParent:GetEffectiveScale()
	my=my/UIParent:GetEffectiveScale()
	local mind,minf=100,nil
	for i,f in ipairs(self:GetDropTargets()) do
		if f._class and f._class=="PlayerFrame" then
			d=f:DistanceTo(mx,my)
			if d==0 then return f,0 end
			if d<mind then
				mind=d
				minf=f
			end
		end
	end	
	return minf,mind
end

function AssignmentFrame:IsCounting()
	return self.countButton:GetChecked()
end
function AssignmentFrame:SetCounting(value)
	return self.countButton:SetChecked(value)
end

function AssignmentFrame:GetDropTargets()
	if self.dropTargets then return self.dropTargets end
	self.dropTargets={}
	local i,s
	for i,s in ipairs(self.uiElements) do
		if s._class and s._class=="PlayerFrame" then
			self.dropTargets[#self.dropTargets+1]=s
		end
	end	
	return self.dropTargets
end

function AssignmentFrame:AssignmentAdded(prevFrame)
	self.prevFrame=prevFrame
	self:UpdatePoints()
end

function AssignmentFrame:ModifyAssignment(str)
	local values=self:GetValues()
	self:ParseAssignmentString(str)
	self:RestoreValues(values)
	Core.planFrame:SaveAssignmentString(self)
	Core.rosterFrame:UpdateUseCounts()
end

function AssignmentFrame:ParseAssignmentString(str)
	local parsed={}
	local l,pos,s,s2
	for l in string.gmatch(str,"([^\n]*)\n?") do
		if #l>0 then 
			if #parsed>0 then
				parsed[#parsed+1]={type="lf"}
			end
			pos=1
			while true do
				s,_=string.find(l,"{",pos,true)
				if s~=nil and s+1<=#l and string.sub(l,s+1,s+1)=="{" then
					parsed[#parsed+1]={type="text",value=string.sub(l,pos,s-1).."{"}
					pos=s+2
				elseif s==pos then
					s2,_=string.find(l,"}",s+1,true)
					if not s2 then break end
					parsed[#parsed+1]=self:ParseAssignmentSegment(string.sub(l,s+1,s2-1))
					pos=s2+1
				elseif s==nil then
					if pos<=#l then
						parsed[#parsed+1]={type="text",value=string.sub(l,pos,#l)}
					end
					break
				else
					parsed[#parsed+1]={type="text",value=string.sub(l,pos,s-1)}
					pos=s
				end
				while #parsed>=2 and parsed[#parsed].type=="text" and parsed[#parsed-1].type=="text" do
					parsed[#parsed-1].value=parsed[#parsed-1].value..parsed[#parsed].value
					table.remove(parsed,#parsed)
				end
			end
		end
	end
	self.assignmentStruct=parsed
	self.assignmentSource=str
	self:ClearAssignmentUI()
	self:BuildAssignmentUI(parsed)
end

function AssignmentFrame:ParseAssignmentSegment(str)
	local ret={type="drop",tab=false,label="",prepend="",append=""}
	local k,v
	for k,v in string.gmatch(str,"([^;=]+)=?([^;]*);?") do
		if k=="label" then ret.label=v 
		elseif k=="tab" then ret.tab=true 
		elseif k=="prepend" then ret.prepend=v
		elseif k=="append" then ret.append=v
		elseif k=="lf" then return {type="msglf"}
		elseif 	k=="star" or k=="circle" or k=="diamond" or k=="triangle" or
				k=="moon" or k=="square" or k=="cross" or k=="skull" or
				k=="rt1" or k=="rt2" or k=="rt3" or k=="rt4" or
				k=="rt5" or k=="rt6" or k=="rt7" or k=="rt8" then
			return {type="text",value="{"..k.."}"}
		end
	end
	return ret
end

function AssignmentFrame:GetValues()
	local i,e,v,pf
	local ret={}
	
	for i,e in ipairs(self:GetDropTargets()) do
		if e.type==PlayerFrame.TYPE_DROP_FULL then
			ret[#ret+1]=e.name
		else
			ret[#ret+1]=""
		end
	end	
	return ret
end

function AssignmentFrame:RestoreValues(values)
	local i,e,v,pf
	local counter=1
	
	for i,e in ipairs(self:GetDropTargets()) do
		if counter>#values then break end
		v=values[counter]
		if v~="" then
			pf=Core.rosterFrame:FindPlayer(v)
			if pf then
				e:Init(pf.name,pf.class,PlayerFrame.TYPE_DROP_FULL,pf.groupnum)
			else
				e:Init(v,"UNKNOWN",PlayerFrame.TYPE_DROP_FULL,0)
			end
		end
		counter=counter+1
	end	
end

function AssignmentFrame:CheckOrigins()
	local i,e,v,pf
		
	for i,e in ipairs(self:GetDropTargets()) do
		if e.type==PlayerFrame.TYPE_DROP_FULL then
			pf=Core.rosterFrame:FindPlayer(e.name)
			if pf then
				e:Init(pf.name,pf.class,PlayerFrame.TYPE_DROP_FULL,pf.groupnum)
			else
				e:Init(e.name,"UNKNOWN",PlayerFrame.TYPE_DROP_FULL,0)
			end
		end
	end	
end

function AssignmentFrame:BuildAssignmentUI(as)
	local prev=nil
	local prevline=nil
	local newline=true
	local prevlinedrop=nil
	local prevlinedropset=false
	local i,j,s,elements,element,usetab
	for i,s in ipairs(as) do
		elements={}
		usetab=false
		
		if s.type=="text" then
			elements[1]=Core:GetFontString()
			elements[1]:SetText(s.value)
		elseif s.type=="drop" then
			if #s.prepend>0 then
				elements[1]=Core:GetFontString()
				elements[1]:SetText(s.prepend)
			end
		
			elements[#elements+1]=Core:GetPlayerFrame(s.label,"INFO",PlayerFrame.TYPE_DROP,0)
			elements[#elements].infoString=s.label
			
			usetab=s.tab
			if not prevlinedropset then
				prevlinedropset=true
				prevlinedrop=elements[#elements]
			end
			
			if #s.append>0 then
				elements[#elements+1]=Core:GetFontString()
				elements[#elements]:SetText(s.append)
			end			
		elseif s.type=="lf" then
			newline=true
		end
		
		for j,element in ipairs(elements) do
			element:SetParent(self)
			element:ClearAllPoints()
			
			if usetab and prevlinedrop and element._class=="PlayerFrame" then
				element:SetPoint("BOTTOMLEFT",prevlinedrop,"BOTTOMLEFT",0,-14)
				prevlinedrop=element
				if newline then
					prevline=element
					newline=false
				end
			elseif newline then
				if prevline then
					element:SetPoint("BOTTOMLEFT",prevline,"BOTTOMLEFT",0,-14)					
				else
					element:SetPoint("TOPLEFT",self,"TOPLEFT",5,-5)
				end
				prevline=element
				newline=false
			else
				element:SetPoint("BOTTOMLEFT",prev,"BOTTOMRIGHT",2,0)
			end
			self.uiElements[#self.uiElements+1]=element
			element:Show()
			prev=element
		end
	end
	self:UpdatePoints()
end

function AssignmentFrame:ClearAssignments()
	local i,e
	for i,e in ipairs(self:GetDropTargets()) do
		if e.type==PlayerFrame.TYPE_DROP_FULL then	
			e:DropTo(nil)
		end
	end
	Core.rosterFrame:UpdateUseCounts()
end

function AssignmentFrame:ClearAssignmentUI()
	local i,s
	for i,s in ipairs(self.uiElements) do
		if s._class and s._class=="PlayerFrame" then
			Core:ReleasePlayerFrame(s)
		else
			Core:ReleaseFontString(s)
		end
	end
	self.dropTargets=nil
	self.uiElements={}
end

function AssignmentFrame:DeleteAssignment()
	Core.planFrame:DeleteAssignment(self)
end

function AssignmentFrame:CreateBaseFrame()
	
--	self.editButton=CreateFrame("Button",nil,self,"UIPanelButtonTemplate")
--	self.editButton:SetParent(self)
--	self.editButton:SetText("Edit")
--	self.editButton:SetPoint("BOTTOMLEFT",self,"BOTTOMLEFT",5,5)
--	self.editButton:SetPoint("TOPRIGHT",self,"BOTTOMLEFT",55,30)
--	self.editButton:SetScript("OnClick",function() self:OpenEdit() end)

	local AceGUI=LibStub("AceGUI-3.0")

	local popupStruct={
		{text=L["Clear assignments"],func=function() self:ClearAssignments() end},
		{text=L["Edit template"],func=function() self:OpenEdit() end},
		{text=L["Move up"],func=function() self:MoveUp() end},
		{text=L["Move down"],func=function() self:MoveDown() end},
		{text=L["Delete template"],func=function() self:DeleteAssignment() end}
	}

	self.menuButton=AceGUI:Create("Button")
	self.menuButton:SetWidth(80)
	self.menuButton:SetText("Menu")
	self.menuButton:SetCallback("OnClick",function()
		EasyMenu(popupStruct, Core.menuFrame, "cursor", 0,0 , "MENU", 0)
	end)	
	self.menuButton.frame:SetParent(self)
	self.menuButton.frame:ClearAllPoints()
	self.menuButton.frame:SetPoint("BOTTOMLEFT",self,"BOTTOMLEFT",5,5)
--	self.menuButton.frame:SetPoint("TOPRIGHT",self,"TOPRIGHT",-5,-5)
	self.menuButton.frame:Show()
	
	self.countLabel=self:CreateFontString(nil,"ARTWORK")
	self.countLabel:SetFontObject(GameFontNormal)
	self.countLabel:SetText("Count roster")
	self.countLabel:SetPoint("RIGHT",self,"RIGHT",-5,0)
	self.countLabel:SetPoint("BOTTOM",self.menuButton.frame,"BOTTOM",0,0)
	self.countLabel:SetPoint("TOP",self.menuButton.frame,"TOP",0,0)
--	self.countLabel:SetPoint("RIGHT",self.countButton,"LEFT",-5,0)	
	
	self.countButton=CreateFrame("CheckButton",nil,self,"UICheckButtonTemplate")
	self.countButton:SetParent(self)
	self.countButton:SetPoint("RIGHT",self.countLabel,"LEFT",0,-1)
--	self.countButton:SetPoint("BOTTOMRIGHT",self,"BOTTOMRIGHT",-5,5)
	self.countButton:SetWidth(20)
	self.countButton:SetHeight(20)
	self.countButton:SetScript("OnClick",function() self:CountToggled() end)
		
	self.channelDrop=CreateFrame("Frame", "OOB_ASSIGN"..self.frameID.."_DROP", self, "UIDropDownMenuTemplate");
	self.channelDrop:SetPoint("TOPLEFT",self.menuButton.frame,"TOPRIGHT",10,2)
--	self.channelDrop:SetPoint("TOPRIGHT",self.menuButton.frame,"BOTTOMRIGHT",20,-5)
	UIDropDownMenu_SetWidth(self.channelDrop, 80)
	UIDropDownMenu_Initialize(self.channelDrop, function() return self:DropDownInit() end)
	
	self.sendButton=AceGUI:Create("Button")
	self.sendButton:SetWidth(80)
	self.sendButton:SetText("Send")
	self.sendButton.frame:SetParent(self)
	self.sendButton.frame:ClearAllPoints()
	self.sendButton.frame:SetPoint("LEFT",self.channelDrop,"RIGHT",5,0)
	self.sendButton.frame:SetPoint("BOTTOM",self.menuButton.frame,"BOTTOM",0,0)
--	self.sendButton.frame:SetPoint("TOPRIGHT",self.menuButton.frame,"BOTTOMRIGHT",0,-35)
	self.sendButton:SetCallback("OnClick",function() self:SendToChat() end)
	self.sendButton.frame:Show()
	
end

function AssignmentFrame:MakeChatMessages()
	local maxLength=200
	local ret={}
	local line=""
	local dropIndex=1
	local dropTargets=self:GetDropTargets()
	local i,s,nextPart
	
	for i,s in ipairs(self.assignmentStruct) do
		nextPart=nil
		if s.type=="text" then
			nextPart=s.value
		elseif s.type=="drop" then
			if dropTargets[dropIndex].type==PlayerFrame.TYPE_DROP_FULL then
				nextPart=""
				if #s.prepend>0 then nextPart=s.prepend end
				nextPart=nextPart..dropTargets[dropIndex].name
				if #s.append>0 then nextPart=nextPart..s.append end
			end
			dropIndex=dropIndex+1
		elseif s.type=="lf" then
			nextPart=" "
		elseif s.type=="msglf" then
			line=string.gsub(line,"^%s+","")
			line=string.gsub(line,"%s+"," ")
			ret[#ret+1]=line			
			line=""
		end
		
		if nextPart then
			next=line..nextPart
			if string.len(next)<=maxLength then
				line=next
			else
				line=string.gsub(line,"^%s+","")
				line=string.gsub(line,"%s+"," ")
				ret[#ret+1]=line
				line=nextPart
			end
		end
	end
	if string.len(line)>0 then
		line=string.gsub(line,"^%s+","")
		line=string.gsub(line,"%s+"," ")
		ret[#ret+1]=line
	end
	return ret
end

function AssignmentFrame:GetChannel()
	return UIDropDownMenu_GetSelectedValue(self.channelDrop)
end

function AssignmentFrame:SetChannel(channel)
	local found=false
	if channel==L["Raid"] or channel==L["Raid warning"] or channel==L["Party"] or
	   channel==L["Guild"] or channel==L["Officer"] or channel==L["Say"] or
	   channel==L["Whisper"] then
		found=true
	else
		for i=5,10 do
			id,name=GetChannelName(i)
			if name==channel then
				found=true
				break
			end
		end
	end
	if not found then channel=L["Raid"] end
	UIDropDownMenu_SetSelectedValue(self.channelDrop,channel)
	UIDropDownMenu_SetText(self.channelDrop,channel)
end

StaticPopupDialogs["OOB_SEND_WHISPER"] = {
  text = L["Send to"],
  button1 = L["Accept"],
  button2 = L["Cancel"],
  OnAccept = nil, -- set in SendToChat
  OnShow = function(self)
    self.editBox:SetText(UnitName("target") or "")
  end,
  timeout = 0,
  whileDead = 1,
  hideOnEscape = 1,
  hasEditBox = 1
};

function AssignmentFrame:SendToChat()
	local channel,type,id,messages,i,m
	channel=UIDropDownMenu_GetSelectedValue(self.channelDrop)
	if channel then
		if channel==L["Raid"] then
			type="RAID"
			channel=""
		elseif channel==L["Party"] then
			type="PARTY"
			channel=""
		elseif channel==L["Say"] then
			type="SAY"
			channel=""
		elseif channel==L["Guild"] then
			type="Guild"
			channel=""
		elseif channel==L["Officer"] then
			type="OFFICER"
			channel=""
		elseif channel==L["Raid warning"] then
			type="RAID_WARNING"
			channel=""
		elseif channel==L["Whisper"] then
			local frame=self
			StaticPopupDialogs["OOB_SEND_WHISPER"].OnAccept=function(self)
				local data=self.editBox:GetText()
				print(data)
--				local data=getglobal(self:GetParent():GetName().."EditBox"):GetText()
				frame:SendToWhisper(data)
			end		
			StaticPopup_Show ("OOB_SEND_WHISPER")
			return
		else
			type="CHANNEL"
			id,_=GetChannelName(channel)
			if not id then return end
			channel=id
		end
	
		messages=self:MakeChatMessages()
		for i,m in ipairs(messages) do
			SendChatMessage(m,type,nil,channel)
		end
	end
end

function AssignmentFrame:SendToWhisper(player)
	local i,m,messages
	messages=self:MakeChatMessages()
	for i,m in ipairs(messages) do
		SendChatMessage(m,"WHISPER",nil,player)
	end
end

function AssignmentFrame:DropDownInit(menu)
	local menu=self.channelDrop
	level=1
	local info

	info= {text = L["Raid"], value = L["Raid"],
	       func = function() self:DropDownClick(L["Raid"]) end }
	UIDropDownMenu_AddButton(info, level)
	info= {text = L["Raid warning"], value = L["Raid warning"],
	       func = function() self:DropDownClick(L["Raid warning"]) end }
	UIDropDownMenu_AddButton(info, level)
	info= {text = L["Party"], value = L["Party"],
	       func = function() self:DropDownClick(L["Party"]) end }
	UIDropDownMenu_AddButton(info, level)
	info= {text = L["Say"], value = L["Say"],
	       func = function() self:DropDownClick(L["Say"]) end }
	UIDropDownMenu_AddButton(info, level)
	info= {text = L["Guild"], value = L["Guild"],
	       func = function() self:DropDownClick(L["Guild"]) end }
	UIDropDownMenu_AddButton(info, level)
	info= {text = L["Officer"], value = L["Officer"],
	       func = function() self:DropDownClick(L["Officer"]) end }
	UIDropDownMenu_AddButton(info, level)
	info= {text = L["Whisper"], value = L["Whisper"],
	       func = function() self:DropDownClick(L["Whisper"]) end }
	UIDropDownMenu_AddButton(info, level)
	
	local i,id,name
	for i=5,10 do
		id,name=GetChannelName(i)
		if id>0 then
			local channel=name
			info = {text = name, value = name,
					func = function() self:DropDownClick(channel) end,
					owner = menu:GetParent() }
			UIDropDownMenu_AddButton(info, level)
		end
	end	
end

function AssignmentFrame:DropDownClick(value)
	UIDropDownMenu_SetSelectedValue(self.channelDrop, value)
end

function AssignmentFrame:CountToggled()
	Core.rosterFrame:UpdateUseCounts()
end

function AssignmentFrame:MoveUp()
	Core.planFrame:MoveAssignmentUp(self)
end

function AssignmentFrame:MoveDown()
	Core.planFrame:MoveAssignmentDown(self)
end

function AssignmentFrame:OpenEdit()
	self.editor:Open(self.assignmentSource)
end

function AssignmentFrame:UpdatePoints()
	self:ClearAllPoints()
	local top
	if self.prevFrame then
		self:SetPoint("TOP",self.prevFrame,"BOTTOM",0,-10)
	else
		self:SetPoint("TOP",self:GetParent(),"TOP",0,-10)
	end
	self:SetPoint("LEFT",self:GetParent(),"LEFT",10,0)
	self:SetPoint("RIGHT",self:GetParent(),"RIGHT",-10,0)
	
	if #self.uiElements > 0 then
		last=self.uiElements[#self.uiElements]
		self:SetHeight(self:GetTop()-last:GetBottom()+35)
	else
		self:SetHeight(50)
	end
end



function AssignmentEditor:New(parent)
	if self ~= AssignmentEditor then return end
	local s=CreateFrame("FRAME",nil,Core.assignmentSF,nil)
	AssignmentEditor:Embed(s)	
	
	s.parentFrame=parent
	s:SetFrameStrata("FULLSCREEN")
	s:SetBackdrop({bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
					edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
					tile=true,tileSize=32,edgeSize=4,insets={left=0,right=0,top=0,bottom=0}})			
	s:Hide()
		
	s.editBox=CreateFrame("EditBox","OOB_A_"..parent.frameID.."_EDITOR_EDITBOX",s,nil)
	s.editBox:SetPoint("TOPLEFT",s,"TOPLEFT",10,-10)
	s.editBox:SetPoint("BOTTOMRIGHT",s,"BOTTOMRIGHT",-10,45)
	s.editBox:SetFontObject(GameFontNormal)
	s.editBox:SetMultiLine(true)
	
	s.saveButton=CreateFrame("Button",nil,s,"UIPanelButtonTemplate")
	s.saveButton:SetText(L["Save"])
	s.saveButton:SetPoint("BOTTOMLEFT",s,"BOTTOMLEFT",10,10)
	s.saveButton:SetPoint("TOPRIGHT",s,"BOTTOMLEFT",70,35)
	s.saveButton:SetScript("OnClick",function () s:Save() end)	

	s.cancelButton=CreateFrame("Button",nil,s,"UIPanelButtonTemplate")
	s.cancelButton:SetText(L["Cancel"])
	s.cancelButton:SetPoint("TOPLEFT",s.saveButton,"TOPRIGHT",10,0)
	s.cancelButton:SetPoint("BOTTOMRIGHT",s.saveButton,"BOTTOMRIGHT",70,0)
	s.cancelButton:SetScript("OnClick",function () s:Cancel() end)	

	s.rootFrameGrip = CreateFrame("Frame", nil, s)
	s.rootFrameGrip:SetWidth(20)
	s.rootFrameGrip:SetHeight(20)
	s.rootFrameGrip:ClearAllPoints()
	s.rootFrameGrip:SetPoint("BOTTOMRIGHT", s, "BOTTOMRIGHT",0,0)	

	s:EnableMouse(true)
	s:SetResizable(true)
	s:SetMovable(true)
	s:SetScript("OnDragStart",self.StartMoving )
	s:SetScript("OnDragStop",
		function()
			s:StopMovingOrSizing()
		end)
	s:RegisterForDrag("LeftButton")
	
	s.rootFrameGrip:EnableMouse(true)
	s.rootFrameGrip:SetScript("OnDragStart", function() s:StartSizing() end )
	s.rootFrameGrip:SetScript("OnDragStop",
		function()
			s:StopMovingOrSizing()
		end)
	s.rootFrameGrip:RegisterForDrag("LeftButton")

	
	
	return s
end

function AssignmentEditor:Cancel()
	self:Hide()
end

function AssignmentEditor:Save()
	self.parentFrame:ModifyAssignment(self.editBox:GetText())
	self:Hide()
end

function AssignmentEditor:Open(text)
	self.editBox:SetText(text)
	self:ClearAllPoints()
	self:SetPoint("CENTER",Core.rootFrame,"CENTER",0,0)
	self:SetWidth(400)
	self:SetHeight(300)
	self:Show()
end
