local ADDON_NAME="OrderOfBattle"

local _G = _G
local pairs = pairs
local ipairs = ipairs
local remove = table.remove
local type = type
local select = select
local next = next

OrderOfBattle = LibStub("AceAddon-3.0"):NewAddon(ADDON_NAME, "AceEvent-3.0", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME, true)

local Core=OrderOfBattle

--SLASH_ORDEROFBATTLE1,SLASH_ORDEROFBATTLE2='/oob','/orderofbattle'
--function SlashCmdList.ORDEROFBATTLE(msg, editbox)
--	Core:ToggleVisibility()
--end

local function tableFind(table,value)
	for i,v in ipairs(table) do
		if v==value then return i end
	end
	return nil
end
Core.tableFind=tableFind

local function tableCopy(table)
	local ret={}
	local i,v
	for i,v in pairs(table) do
		if type(v)=="table" then
			ret[i]=tableCopy(v)
		else
			ret[i]=v
		end
	end
	return ret
end

local function tableShuffle(table)
	local temp,i,j
	for i=1,#table do
		j=random(#table-i+1)+i-1
		temp=table[i]
		table[i]=table[j]
		table[j]=temp
	end
end


local options = {
    name = ADDON_NAME,
    handler = Core,
    type = 'group',
	get = function(info) return Core:GetOptionParam(info) end,
	set = function(info,val) Core:SetOptionParam(info,val) end,
    args = {
		general = {
			type="group",
			name= L["General"],
			args={
				enable = {
					type = "toggle",
					name = L["Enable"],
					desc = L["Enable or disable addon"],
					width = "full",
					order = 0,
					set = function(info, v)
						Core:SetOptionParam(info,v)
						if v then
							Core:DoEnable()
						else
							Core:DoDisable()
						end
					end,
				},
				simulate = {
					type = "toggle",
					name = L["Simulate"],
					desc = L["Simulate a raid..."],
					width = "full",
					order = 1,
					set = function(info, v)
						Core:SetOptionParam(info,v)
						Core.simulate=v
						Core:RaidChanged()
					end,
				},
				showonload = {
					type = "toggle",
					name = L["Show on load"],
					desc = L["Show main frame on load"],
					width = "full",
					order = 2,
					set = function(info, v)
						Core:SetOptionParam(info,v)
					end,
				},
				rosterswaps = {
					type = "toggle",
					name = L["Enable raid swaps"],
					desc = L["Enable swapping player groups in roster frame"],
					width = "full",
					order = 3,
					set = function(info, v)
						Core:SetOptionParam(info,v)
					end,
				}
			}
		},
    },
}
local optionsSlash = {
	type = "group",
	name = L["Slash command"],
	order = -3,
	get = function(info) return Core:GetOptionParam(info) end,
	set = function(info,val) Core:SetOptionParam(info,val) end,
	args = {
		show = {
			type = "execute",
			name = L["Show"],
			desc = L["Show main frame"],
			func =	function()
						Core:ShowRootFrame()
					end
		},
		hide = {
			type = "execute",
			name = L["Hide"],
			desc = L["Hide main frame"],
			func =	function()
						Core:HideRootFrame()
					end
		},
		toggle = {
			type = "execute",
			name = L["Toggle"],
			desc = L["Toggle main frame"],
			func =	function()
						Core:ToggleVisibility()
					end
		},
		options = {
			type = "group",
			name = L["Options"],
			desc = L["Addon options"],
			args = options.args
		}
	}
}


local demoplans = {
			{
				name="Leviathan",
				assignments={
					{str="G1 G2 - 1 demo driver, 1 demo gunner, 1 siege driver, 1 siege gunner, 1 siege gunner for G3/G4",
						values={},channel="Raid",count=false},
					{str="G3 G4 - 1 demo driver, 1 siege driver, 1 bike, 2 launched demo passengers",
						values={},channel="Raid",count=false},
					{str="G5 - 1 demo driver, 1 demo gunner, 1 siege driver, 1 siege gunner, 1 bike",
						values={},channel="Raid",count=false},
					{str="Launched people are {} + {}, {} + {}",
						values={},channel="Raid",count=false},
					{str="Siege engines 3+4 interrupting",
						values={},channel="Raid",count=false},
					{str="G1 sw, G2 se, G3 ne, G4 se, G5 ne",
						values={},channel="Raid",count=false},
				}
			},
			{
				name="XT-002",
				assignments={
					{str="{} north, {} east, {} south",values={},channel="Raid",count=false}
				}
			},
			{
				name="Iron council",
				assignments={
					{str="Tanks: {} Steelbreaker, {} Molgeim", values={},channel="Raid",count=false},
					{str="{} dispel fusion punch",values={},channel="Raid",count=false},
					{str="After Steelbreaker dies {} interrupt whirl, {} chain",values={},channel="Raid",count=false},
					{str="Ranged {label=at least 3} {} {} {} {}", values={},channel="Raid",count=false},
					{str="Cooldowns in p3\n2nd tank: {} {}, {} {}, {} {}\n3rd tank: {tab} {}, {} {}, {} {},\n{tab} {}",
						values={},channel="Raid",count=false},
					{str="Remember soulstone/res!",values={},channel="Raid",count=false},
				}
			},
			{
				name="Freya x3",
				assignments={
					{
						str="Ranged positions\nN    {label=healer} {} {} {} -\nNE {label=healer;tab} {} {} {} -\nSE {label=healer;tab} {} {} {} -\nSW {label=healer;tab} {} {} {} -\nNW {label=healer;tab} {} {} {}",
						values={},channel="Raid",count=false,
					},
					{
						str="Roots    {label=x5 burst} {} {} {} {} -\nTrees {label=x4;tab} {} {} {} {}",
						values={},channel="Raid",count=false,
					},
					{
						str="Tanks {label=warrior} Freya+stormlasher - {label=dk} snaplasher - {label=dps bear} water ele.",
						values={},channel="Raid",count=false,
					},
					{
						str="{label=mel, inter} {label=mel} on water elemental -\n{label=mel, inter} {label=mel} {} {} {} on stormlasher -\nRanged on snaplasher",
						values={},channel="Raid",count=false,
					},
					
				}
			},
			{
				name="Thorim",
				assignments={
					{
						str="Gauntlet {label=tank} {label=healer} {label=healer} {label=ranged dps} {}\n{tab} {} {} {} {}",
						values={},channel="Raid",count=false,
					},
					{
						str="P2 ranged\nNW {label=healer} {} {} {} -\nNE {label=healer;tab} {} {} {} -\nSE {label=healer;tab} {} {} {} -\nSW {label=healer;tab} {} {} {} -\nW {label=healer;tab} {} {} {}",
						values={},channel="Raid",count=false,
					},
					{
						str="P2 melee\nLeft   {} {} {} {} {} {} -\nRight {tab} {} {} {} {} {}",
						values={},channel="Raid",count=false,
					},
					{
						str="P2 Tank {} - Mind control {} {}",
						values={},channel="Raid",count=false,
					},
					{
						str="Cooldowns charge\n7    {} {} - 8  {} {} -\n9 {tab} {} - 10 {} {} -\n11 {tab} {} - 12 {} {}",
						values={},channel="Raid",count=false,
					}
				}
			},
			{
				name="Mimiron",
				assignments={
					{str="Plasma cooldowns {} {} {} {}",values={},channel="Raid",count=false},
					{str="{} north, {} se, {} sw",values={},channel="Raid",count=false},
					{str="P3 {label=ranged dps} tanking head, {} bombs, {} doing cores",values={},channel="Raid",count=false}
				}
			},
			{
				name="Yogg Saron",
				assignments={
					{
						str="Portals - {label=healer} south,\n{tab} {} {} west,\n{tab} {} {} north,\n{tab} {} {} east.",
						values={},channel="Raid",count=false,
					},
					{
						str="Illusion - west go left, north go far, east go right.",
						values={},channel="Raid",count=false,
					}
				}
			}
		}


local defaults = {
	profile = {
		enable = true,
		simulate = false,
		showonload = false,
		rosterswaps = true,
		root_point = {
			"CENTER","CENTER",0,0
		},
		root_size = {
			800,600
		},
		plans = {},
		first_run = true,
	}
}


function Core:SetOptionParam(info,value,folder)
	local varName = nil
	if type(info) == "string" then
		varName = info
	else
		varName = info[#info]
	end
	if not folder then folder=self.db.profile end
	folder[varName] = value
end
function Core:GetOptionParam(info,folder)
	local varName = nil
	if type(info) == "string" then
		varName = info
	else
		varName = info[#info]
	end
	if not folder then folder=self.db.profile end
	return folder[varName]
end

function Core.Embed(self, other)
	local k,v
	if _G.getmetatable(other) then
		for k, v in pairs(self) do
			other[k] = v
		end
	else
		_G.setmetatable(other, {__index = self})
	end
	return other
end


function Core:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New(ADDON_NAME.."_DB",defaults)

	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfig-3.0"):RegisterOptionsTable(ADDON_NAME, options, {"orderofbattle", "oob"})
	LibStub("AceConfig-3.0"):RegisterOptionsTable(ADDON_NAME.."SlashCommand", optionsSlash, {"orderofbattle", "oob"})
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(ADDON_NAME, options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions(ADDON_NAME, ADDON_NAME)
	
	if self.db.profile.first_run then
		self.db.profile.first_run=false
		self:SetDemoPlans()
	end
	
	self.selectedFrames={}
	
	self.playerFrameCache={}
	self.assignmentFrameCache={}
	self.dropFrameCache={}
	self.fontStringCache={}
	
	LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Order of Battle", {
		type = "launcher",
		icon = "Interface\\Icons\\INV_Misc_Map_01",
		OnClick = function(clickedframe, button)
			if button=="LeftButton" then
				self:ToggleVisibility()
			else
				self:OpenLDBMenu()
			end
		end,
	})
	
end

function Core:OpenLDBMenu()
	local i,p,a
	
	if not self.ldbmenu then
		self.ldbmenu = CreateFrame("Frame", "OOB_LDB_MENU")
	end
	local ldbmenu = self.ldbmenu
	
	local selectPlan=function(_,p,i)
		UIDropDownMenu_SetSelectedValue(Core.planDrop, i)
		UIDropDownMenu_SetText(Core.planDrop, p.name)
		self.planFrame:SaveValues()
		self.planFrame:SelectPlan(p)
		HideDropDownMenu(1)
	end
	local send=function(_,a)
		a:SendToChat()
	end

	ldbmenu.displayMode = "MENU"
	local info={}
	ldbmenu.initialize=function(self,level)
		if not level then return end
	    if level == 1 then
	        info={text=(Core.visible and L["Hide dialog"] or L["Show dialog"]),func=function() Core:ToggleVisibility() end}
	        UIDropDownMenu_AddButton(info, level)
			
			info={text=(Core.planFrame.planStruct and Core.planFrame.planStruct.name or L["Select plan"]),hasArrow=true,value="plans"}
	        UIDropDownMenu_AddButton(info, level)
			
			if Core.planFrame.planStruct then
				info={isTitle = 1,text = L["Send assignments"]}
				UIDropDownMenu_AddButton(info, level)
				local i,a
				for i,a in ipairs(Core.planFrame.planStruct.assignments) do
					info={text=string.sub(Core.planFrame.assignmentFrames[i]:MakeChatMessages()[1],1,20),func=send,arg1=Core.planFrame.assignmentFrames[i]}
					UIDropDownMenu_AddButton(info, level)
				end
			end
		elseif level == 2 then
		    if UIDROPDOWNMENU_MENU_VALUE == "plans" then
				local i,p
				for i,p in ipairs(Core.db.profile.plans) do
					info={text=p.name,func=selectPlan,arg1=p,arg2=i,checked=(Core.planFrame.planStruct==p and true or nil)}
					UIDropDownMenu_AddButton(info, level)
				end
			end
		end
	end
	local x,y = GetCursorPosition(UIParent);
	ToggleDropDownMenu(1, nil, ldbmenu, "UIParent", x / UIParent:GetEffectiveScale() , y / UIParent:GetEffectiveScale())
end

function Core:SetDemoPlans()
	self.db.profile.plans=tableCopy(demoplans)
end

function Core:OnEnable()
	self:DoEnable()
end
function Core:DoEnable()
	if not self.db.profile.enable then return end
	
	if not self.rootFrame then
		self:CreateRootFrame()
	else
		self:EnableRootFrame()
	end
	
	self:RegisterEvent("PARTY_MEMBERS_CHANGED","RaidChanged")	
	self:RegisterEvent("PARTY_MEMBER_DISABLE","RaidChanged")	
	self:RegisterEvent("PARTY_MEMBER_ENABLE","RaidChanged")	
	
	self.simulate=self.db.profile.simulate
	self:RaidChanged()
end

function Core:OnDisable()
	self:DoDisable()
end
function Core:DoDisable()
	self.rootFrame:Hide()
	self.rootFrame:SetScript("OnUpdate",nil )
	self:UnregisterAllEvents()
	self:SaveDialogPos()
end

function Core:CreateRootFrame()
	local AceGUI=LibStub("AceGUI-3.0")
	self.dialog = AceGUI:Create("Frame")
	self.dialog:SetTitle(L["Order of Battle"])
	self.dialog.frame:SetFrameStrata("DIALOG")
	
	self.rootFrame=CreateFrame("FRAME",ADDON_NAME.."_RootFrame",self.dialog.content)
	local point=self.db.profile.root_point
	local size=self.db.profile.root_size
	
	self.rootFrame:SetPoint("TOPLEFT",self.dialog.content,"TOPLEFT",0,0)
	self.rootFrame:SetPoint("BOTTOMRIGHT",self.dialog.content,"BOTTOMRIGHT",0,0)
	self.dialog.frame:ClearAllPoints()
	self.dialog.frame:SetPoint(point[1],UIParent,point[2],point[3],point[4])
	self.dialog.frame:SetWidth(size[1])
	self.dialog.frame:SetHeight(size[2])

	
	self.rosterSF=CreateFrame("ScrollFrame","OOB_ROSTERSF", self.rootFrame, "UIPanelScrollFrameTemplate")
	self.rosterSF:SetPoint("TOPRIGHT",self.rootFrame,"TOPRIGHT",-40,-30)
	self.rosterSF:SetPoint("BOTTOMLEFT",self.rootFrame,"BOTTOMRIGHT",-140,10)
	self.rosterFrame=OOBRosterFrame:New(self.rosterSF) 
	self.rosterSF:SetScrollChild(self.rosterFrame)
	self.rosterFrame:SetPoint("TOPLEFT",self.rosterSF,"TOPLEFT",0,0)
	
	self.planDrop=CreateFrame("Frame", "OOB_PLAN_DROP", self.rootFrame , "UIDropDownMenuTemplate");
	self.planDrop:SetPoint("TOPLEFT",self.rootFrame,"TOPLeft",5,-5)
	UIDropDownMenu_SetWidth(self.planDrop, 150)
	UIDropDownMenu_Initialize(self.planDrop, function() return self:PlanDropInit() end)	

	self.assignmentSF=CreateFrame("ScrollFrame","OOB_ASSIGNMENTSF", self.rootFrame, "UIPanelScrollFrameTemplate")
	self.assignmentSF:SetPoint("BOTTOM",self.rootFrame,"BOTTOM",0,10)
	self.assignmentSF:SetPoint("LEFT",self.rootFrame,"LEFT",0,0)
	self.assignmentSF:SetPoint("TOP",self.rootFrame,"TOP",0,-30)
	self.assignmentSF:SetPoint("RIGHT",self.rosterFrame,"LEFT",-30,0)

	self.planFrame=OOBPlanFrame:New(self.assignmentSF)
	self.assignmentSF:SetScrollChild(self.planFrame)
	self.planFrame:ClearAllPoints()
	self.planFrame:SetPoint("TOPLEFT",self.assignmentSF,"TOPLEFT",0,0)
	self.planFrame:SetWidth(max(self.assignmentSF:GetWidth(),600))	
	self.planFrame:SetHeight(20)	
	self.assignmentSF:SetScript("OnSizeChanged",function()
		self.planFrame:SetWidth(max(self.assignmentSF:GetWidth(),600))
	end)


	
	self.dialog:SetCallback("OnClose", function(widget) self:HideRootFrame(true) end)
		
	local popupStruct={
		{text=L["New plan"],func=function() self:NewPlan() end},
		{text="",disable=true,notClickable=1,func=function() end},
		{text=L["Rename plan"],func=function() self:RenamePlan() end},
		{text=L["New template"],func=function() self.planFrame:NewAssignment() end},
		{text=L["Move plan up"],func=function() self:MovePlanUp() end},
		{text=L["Move plan down"],func=function() self:MovePlanDown() end},
		{text="",disable=true,notClickable=1,func=function() end},
		{text=L["Delete plan"],func=function() self:DeletePlan() end}
	}

	self.menuButton=AceGUI:Create("Button")
	self.menuButton:SetWidth(80)
	self.menuButton:SetText(L["Menu"])
	self.menuButton:SetCallback("OnClick",function()
		EasyMenu(popupStruct, self.menuFrame, "cursor", 0,0 , "MENU", 0)
	end)	
	self.menuButton.frame:SetParent(self.rootFrame)
	self.menuButton.frame:SetPoint("TOPLEFT",self.rootFrame,"TOPLEFT",200,-5)
	self.menuButton.frame:Show()
	
	self.menuFrame = CreateFrame('Frame', 'OOB_POPUP_MENU_FRAME', UIParent, "UIDropDownMenuTemplate")
	
	self:EnableRootFrame()
	
end

function Core:SaveDialogPos()
	local point, relativeTo, relativePoint, xOfs, yOfs = self.dialog.frame:GetPoint(1)
	self.db.profile.root_point={point,relativePoint,xOfs,yOfs}
	local w=self.dialog.frame:GetWidth()
	local h=self.dialog.frame:GetHeight()
	self.db.profile.root_size={w,h}
end

function Core:EnableRootFrame()	
	if not self.db.profile.showonload then 
		self.dialog:Hide()
		self.visible=false
	else
		self.dialog:Show()
		self.visible=true
	end
end

function Core:ShowRootFrame()
	self.dialog:Show()
	self.visible=true
	self:OnUpdate()
end
function Core:HideRootFrame(fromhandler)
	if fromhandler==nil then fromhandler=false end
	if fromhandler then
		self.visible=false
		self.planFrame:SaveValues()
		self:SaveDialogPos()
	else
		self.dialog:Hide()
	end
end

function Core:ToggleVisibility()
	if self.visible then
		self:HideRootFrame()
	else
		self:ShowRootFrame()
	end
end


function Core:PlanDropInit()
	local menu=self.planDrop
	level=1
	local info = UIDropDownMenu_CreateInfo()
	local i,p
	
	for i,p in ipairs(self.db.profile.plans) do
		info.text = p.name
		info.value = i
		info.func = function() self:PlanDropClick(i) end
		info.owner = menu:GetParent()
		info.checked = nil
		info.icon = nil
		UIDropDownMenu_AddButton(info, level)
	end	

end

function Core:PlanDropClick(value)
	UIDropDownMenu_SetSelectedValue(self.planDrop, value)
	self.planFrame:SaveValues()
	self.planFrame:SelectPlan(self.db.profile.plans[value])
end

function Core:NewPlan()
	self.db.profile.plans[#self.db.profile.plans+1]={name="New Plan",assignments={}}
end

function Core:DeletePlan()
	local i,p
	local found=0
	if self.planFrame.planStruct then
		for i,p in ipairs(self.db.profile.plans) do
			if p==self.planFrame.planStruct then
				found=i
				break
			end
		end
		if found>0 then
			table.remove(self.db.profile.plans,found)
			UIDropDownMenu_SetSelectedValue(self.planDrop, 0)
			UIDropDownMenu_SetText(self.planDrop, "")
			self.planFrame:ClearPlan()
		end
	end
end


StaticPopupDialogs["OOB_RENAME_PLAN"] = {
  text = L["Enter new name for plan"],
  button1 = L["Accept"],
  button2 = L["Cancel"],
  OnAccept = function(self)
    local data=self.editBox:GetText()
	Core:DoRename(data)
  end,
  OnShow = function(self)
    self.editBox:SetText(Core.renameData)
  end,
  timeout = 0,
  whileDead = 1,
  hideOnEscape = 1,
  hasEditBox = 1
};

function Core:RenamePlan()
	if self.planFrame.planStruct then
		self.renameData=self.planFrame.planStruct.name
		StaticPopup_Show ("OOB_RENAME_PLAN")
	end
end
function Core:DoRename(data)
	if self.planFrame.planStruct then
		self.planFrame.planStruct.name=data
		UIDropDownMenu_SetText(self.planDrop, data)
	end
end

function Core:MovePlanUp()
	local i,p
	local found=0
	if self.planFrame.planStruct then
		for i,p in ipairs(self.db.profile.plans) do
			if p==self.planFrame.planStruct then
				found=i
				break
			end
		end
		if found>1 then
			local temp=self.db.profile.plans[found-1]
			table.remove(self.db.profile.plans,found-1)
			table.insert(self.db.profile.plans,found,temp)
			UIDropDownMenu_SetSelectedValue(self.planDrop, found-1)
		end
	end
end

function Core:MovePlanDown()
	local i,p
	local found=0
	if self.planFrame.planStruct then
		for i,p in ipairs(self.db.profile.plans) do
			if p==self.planFrame.planStruct then
				found=i
				break
			end
		end
		if found<#self.db.profile.plans then
			local temp=self.db.profile.plans[found]
			table.remove(self.db.profile.plans,found)
			table.insert(self.db.profile.plans,found+1,temp)
			UIDropDownMenu_SetSelectedValue(self.planDrop, found+1)
		end
	end
end

function Core:GetAllAssignmentFrames()
	return self.planFrame:GetAllAssignmentFrames()
end

function Core:FindMouseTarget()
	return self.planFrame:FindMouseTarget()
end

function Core:ClearSelection()
	local i,f
	for i,f in ipairs(self.selectedFrames) do
		f:Deselect(true)
	end
	self.selectedFrames={}
end

function Core:SelectFrame(f)
	self.selectedFrames[#self.selectedFrames+1]=f
end
function Core:SelectionNames()
	local ret={}
	local i,v
	for i,v in ipairs(self.selectedFrames) do
		ret[#ret+1]=v.name
	end
	return ret
end

function Core:DeselectFrame(f)
	local i,f2
	for i,f2 in ipairs(self.selectedFrames) do
		if f==f2 then
			table.remove(self.selectedFrames,i)
			break
		end
	end
end

function Core:GetSelectedFrames()
	return self.selectedFrames
end


local simRaid={
	{L["Tank"]..1,"WARRIOR",1,true,false},
	{L["Tank"]..2,"DRUID",1,true,false},
	{L["Tank"]..3,"DEATHKNIGHT",1,true,false},
	{L["Healer"]..1,"PALADIN",1,true,false},
	{L["Melee"]..1,"DRUID",1,true,false},
	
	{L["Melee"]..2,"DRUID",2,true,false},
	{L["Melee"]..3,"SHAMAN",2,true,false},
	{L["Melee"]..4,"DEATHKNIGHT",2,true,false},
	{L["Melee"]..5,"PALADIN",2,true,false},
	{L["Melee"]..6,"ROGUE",2,true,false},
	
	{L["Ranged"]..1,"MAGE",3,true,false},
	{L["Ranged"]..2,"SHAMAN",3,true,false},
	{L["Ranged"]..3,"WARLOCK",3,true,false},
	{L["Healer"]..2,"DRUID",3,true,false},
	{L["Healer"]..3,"PRIEST",3,true,false},
	
	{L["Ranged"]..4,"MAGE",4,true,false},
	{L["Ranged"]..5,"SHAMAN",4,true,false},
	{L["Ranged"]..6,"WARLOCK",4,true,false},
	{L["Healer"]..4,"DRUID",4,true,false},
	{L["Healer"]..5,"PRIEST",4,true,false},

	{L["Ranged"]..7,"HUNTER",5,true,false},
	{L["Ranged"]..8,"HUNTER",5,true,false},
	{L["Ranged"]..9,"PRIEST",5,true,false},
	{L["Ranged"]..10,"WARLOCK",5,true,false},
	{L["Healer"]..6,"PALADIN",5,true,false},
	
	{L["Reserve"]..1,"MAGE",6,true,false},
	{L["Reserve"]..2,"SHAMAN",6,true,false},
	{L["Reserve"]..3,"WARRIOR",6,true,false},

	{L["Reserve"]..4,"PRIEST",7,true,false},
	{L["Reserve"]..5,"DRUID",7,true,false},
	
	{L["Reserve"]..6,"DEATHKNIGHT",8,true,false},
	{L["Reserve"]..7,"DEATHKNIGHT",8,true,false},
}

function Core:SimRaidRemove(player)
	local i,p
	for i,p in ipairs(simRaid) do
		if p[1]==player then
			table.remove(simRaid,i)
			self:RaidChanged()
			return true
		end
	end
	return false
end

function Core:SimRaidAdd(player,class,group)
	local groupcount=0
	for i,p in ipairs(simRaid) do
		if p[3]==group then
			groupcount=groupcount+1
		end
	end
	if groupcount<5 then
		simRaid[#simRaid+1]={player,class,group,true,false}
	end
	self:RaidChanged()	
end

function Core:SimRaidSwap(player1,player2)
	if player1>0 and player2>0 and player1<=#simRaid and player2<=#simRaid then
		local temp=simRaid[player1][3]
		simRaid[player1][3]=simRaid[player2][3]
		simRaid[player2][3]=temp
		self:RaidChanged()	
	end
end

function Core:SimRaidMove(player,group)
	local groupcount=0
	for i,p in ipairs(simRaid) do
		if p[3]==group then
			groupcount=groupcount+1
		end
	end
	if groupcount<5 and player>0 and player<=#simRaid then
		simRaid[player][3]=group
		self:RaidChanged()	
	end
end

function Core:RaidChanged()
	self.updateRaid=true
	if self.visible or self.simulate then 
		self.rootFrame:SetScript("OnUpdate",function() self:OnUpdate() end )
	end
end

function Core:RaidChangedNow()
	self.updateRaid=false
	if self.rosterFrame:UpdatePlayers() then
		self.planFrame:CheckOrigins()
		self.rosterFrame:UpdateUseCounts()
	end
end

function Core:OnUpdate()
	if self.visible and self.updateRaid then
		self:RaidChangedNow()
	end
	if self.simulate then
--		if random(40)==1 then
--			print("simulate raid change")
--			tableShuffle(simRaid)
--			self:RaidChanged()
--		end
	else
		self.rootFrame:SetScript("OnUpdate",nil )
	end
end

function Core:FindPlayerRaidID(name)
	local i,iname
	for i=1,40 do
		iname,_,_,_,_=self:GetRaidInfo(i)
		if iname==name then
			return i
		end
	end
	return nil
end

function Core:GetRaidInfo(i)
	if self.simulate then
		if i<=#simRaid then
			return simRaid[i][1],simRaid[i][2],simRaid[i][3],simRaid[i][4],simRaid[i][5]
		else
			return nil
		end
	else
		if GetNumRaidMembers()>0 then
			local name, rank, subgroup, level, class, fileName, 
				zone, online, isDead, role, isML = GetRaidRosterInfo(i);
			
			return name,fileName,subgroup,online,isDead
		else
			if i>5 then return nil end
			local name,fileName,online,isDead
			name=UnitName("party"..i)
			if name==nil then
				if i==1 or UnitName("party"..(i-1))~=nil then
					name=UnitName("player")
					_,fileName=UnitClass("player")
					online=1
					isDead=UnitIsDeadOrGhost("player")
				else
					return nil
				end
			else
				_,fileName=UnitClass("party"..i)
				online=UnitIsConnected("party"..i)
				isDead=UnitIsDeadOrGhost("party"..i)			
			end
			return name,fileName,1,online,isDead
		end
	end
end


function Core:GetPlayerFrame(name,class,type,groupnum)
	local frame
	frame=remove(self.playerFrameCache)
	if not frame then
		frame=OOBPlayerFrame:New(self.rootFrame)
	end
	if name then
		frame:Init(name,class,type,groupnum)
	end
	return frame
end

function Core:ReleasePlayerFrame(frame)
	frame:Hide()
	frame.type=OOBPlayerFrame.TYPE_INVALID
	self.playerFrameCache[#self.playerFrameCache+ 1]=frame
end

function Core:GetAssignmentFrame()
	local frame=remove(self.assignmentFrameCache)
	if not frame then
		frame=AssignmentFrame:New(self.planFrame)
	end
	return frame
end

function Core:ReleaseAssignmentFrame(frame)
	frame:Hide()
	self.assignmentFrameCache[#self.assignmentFrameCache+ 1]=frame
end

function Core:GetFontString()
	local f=remove(self.fontStringCache)
	if not f then
		f=self.rootFrame:CreateFontString(nil,"ARTWORK")
		f:SetFontObject(GameFontNormal)
	end
	f:Show()
	return f
end

function Core:ReleaseFontString(f)
	f:Hide()
	self.fontStringCache[#self.fontStringCache+ 1]=f
end
