local _G = _G
local pairs = pairs
local ipairs = ipairs
local remove = table.remove
local type = type
local select = select
local next = next

local Core=OrderOfBattle
local PlayerFrame=OOBPlayerFrame

OOBPlanFrame = {Embed = Core.Embed}
local PlanFrame = OOBPlanFrame

function PlanFrame:New(parent)
	if self ~= PlanFrame then return end
	local s=CreateFrame("FRAME",nil,parent,nil)
	PlanFrame:Embed(s)	
	
	s.assignmentFrames={}
	
	s.planStruct=nil
	
	return s;
end
	
function PlanFrame:SaveValues()
	if self.planStruct then
		for i,a in ipairs(self.planStruct.assignments) do
			a.values=self.assignmentFrames[i]:GetValues()
			a.channel=self.assignmentFrames[i]:GetChannel()
			a.count=self.assignmentFrames[i]:IsCounting()
		end
	end
end

function PlanFrame:SelectPlan(planStruct)
	self:ClearAssignmentFrames()
	local i,a,f,h
	h=0
	for i,a in ipairs(planStruct.assignments) do
		f=self:NewAssignmentFrame()
		f:ParseAssignmentString(a.str)
		f:RestoreValues(a.values)
		f:SetChannel(a.channel)
		f:SetCounting(a.count)
		f:Show()
		h=self:GetTop()-f:GetBottom()
	end
	self:SetHeight(h+20)
	self.planStruct=planStruct
	Core.rosterFrame:UpdateUseCounts()
end

function PlanFrame:ClearAssignmentFrames()
	local i,f
	for i,f in ipairs(self.assignmentFrames) do
		f:Dispose()
		Core:ReleaseAssignmentFrame(f)
	end
	self.assignmentFrames={}
end

function PlanFrame:ClearPlan()
	self:ClearAssignmentFrames()
	self.planStruct=nil
end

function PlanFrame:MoveAssignmentUp(frame)
	local i,p
	local found=0
	if self.planStruct then
		for i,f in ipairs(self.assignmentFrames) do
			if f==frame then
				found=i
				break
			end
		end
		if found>1 then
			self:SaveValues()
			local temp=self.planStruct.assignments[found-1]
			table.remove(self.planStruct.assignments,found-1)
			table.insert(self.planStruct.assignments,found,temp)
			self:SelectPlan(self.planStruct)
		end
	end
end

function PlanFrame:MoveAssignmentDown(frame)
	local i,p
	local found=0
	if self.planStruct then
		for i,f in ipairs(self.assignmentFrames) do
			if f==frame then
				found=i
				break
			end
		end
		if found<#self.planStruct.assignments then
			self:SaveValues()
			local temp=self.planStruct.assignments[found]
			table.remove(self.planStruct.assignments,found)
			table.insert(self.planStruct.assignments,found+1,temp)
			self:SelectPlan(self.planStruct)
		end
	end
end

function PlanFrame:CheckOrigins()
	local i,f
	for i,f in ipairs(self.assignmentFrames) do
		f:CheckOrigins()
	end
end

function PlanFrame:NewAssignmentFrame()
	local prev=nil
	local frame
	if #self.assignmentFrames > 0 then prev=self.assignmentFrames[#self.assignmentFrames] end
	frame=Core:GetAssignmentFrame()
	frame:AssignmentAdded(prev)
	self.assignmentFrames[#self.assignmentFrames+1]=frame
	return frame
end

function PlanFrame:NewAssignment()
	if self.planStruct then
		self:SaveValues()
		self.planStruct.assignments[#self.planStruct.assignments+1]={str="",values={}}
		self:SelectPlan(self.planStruct)
	end
end

function PlanFrame:DeleteAssignment(aframe)
	self:SaveValues()
	local i,f
	for i,f in ipairs(self.assignmentFrames) do
		if f==aframe then
			table.remove(self.planStruct.assignments,i)
			self:SelectPlan(self.planStruct)
			break
		end
	end
end

function PlanFrame:SaveAssignmentString(aframe)
	local i,f
	self:SaveValues()
	for i,f in ipairs(self.assignmentFrames) do
		if f==aframe then
			self.planStruct.assignments[i].str=aframe.assignmentSource
			break
		end
	end	
end

function PlanFrame:GetAllAssignmentFrames()
	return self.assignmentFrames
end

function PlanFrame:FindMouseTarget()
	local i,f
	local t,d,mint,mind,minf=nil,nil,nil,20,nil
	for i,f in ipairs(self.assignmentFrames) do
		t,d=f:FindMouseTarget()
		if d==0 then
			return t,d,f
		elseif d<mind then
			mind=d
			mint=t
			minf=f
		end
	end
	return mint,mind,minf
end