local _G = _G
local pairs = pairs
local ipairs = ipairs
local remove = table.remove
local type = type
local select = select
local next = next

local Core=OrderOfBattle

OOBPlayerFrame = {Embed = Core.Embed, _class="PlayerFrame"}
local PlayerFrame=OOBPlayerFrame

PlayerFrame.TYPE_ROSTER=0
PlayerFrame.TYPE_DROP=1
PlayerFrame.TYPE_DROP_FULL=2
PlayerFrame.TYPE_ROSTER_GROUP=3
PlayerFrame.TYPE_DRAGGING=4
PlayerFrame.TYPE_INVALID=5

PlayerFrame.dragging=false
PlayerFrame.dragFrames=nil
PlayerFrame.dragUpdatePending=false

function PlayerFrame:New(parent)
	if self ~= PlayerFrame then return end
	local s=CreateFrame("BUTTON",nil,parent,nil)
	PlayerFrame:Embed(s)	
	
	s:ClearAllPoints()
	s:SetPoint("TOPLEFT",parent,"TOPLEFT",0,0);
	s:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
					edgeFile = nil, edgeSize = 0 ,
					tile = true, tileSize = 16, 
					insets = { left = 0, right = 0, top = 0, bottom = 0 }})
	s:SetBackdropColor(0.5, 0.5, 0.5, 1)	
	s:SetHighlightTexture("Interface\\Buttons\\UI-Listbox-Highlight.blp","ADD")
	
	s.text = s:CreateFontString(nil, "ARTWORK")
	s.text:SetFont("Fonts\\FRIZQT__.TTF",10)
--	s.text:SetFontObject(GameFontNormal)
	s.text:SetJustifyH("LEFT")
	s.text:SetPoint("TOPLEFT",s,"TOPLEFT",6,1)
    
	s:SetWidth(80)
	s:SetHeight(11)
    
	s:SetMovable(true)
	s:RegisterForDrag("LeftButton")
	s:EnableMouse(true)
	s:SetScript("OnDragStart", s.EventDragStart)
	s:SetScript("OnDragStop", s.EventDragStop )
	s:SetScript("OnClick", s.EventClick)
	s:SetScript("OnMouseDown", s.EventMouseDown)
	s:SetScript("OnMouseUp", s.EventMouseUp)
	return s
end

function PlayerFrame:Init(name,class,type,groupnum)
	self.name=name
	self.groupnum=groupnum
	self.class=class
	self.type=type
	self.selected=false
	self:UnlockHighlight()
	self.error=false
	self.errorMessage=""
	self:SetBackdropColor(0.5, 0.5, 0.5, 1)	
	self.useCount=0
	self.groupMembers={}
	self:UpdateText()
	self:Show()
end

function PlayerFrame:DistanceTo(x,y)
	if x>=self:GetLeft() and x<=self:GetRight() then
		if y>=self:GetBottom() and y<=self:GetTop() then
			return 0
		end
		return min(abs(y-self:GetBottom()),abs(y-self:GetTop()))
	elseif y>=self:GetBottom() and y<=self:GetTop() then
		return min(abs(x-self:GetLeft()),abs(x-self:GetRight()))
	else
		return min(abs(y-self:GetBottom()),abs(y-self:GetTop()))+min(abs(x-self:GetLeft()),abs(x-self:GetRight()))
	end
end

function PlayerFrame:SetError(error,message)
	self.error=error
	self.errorMessage=message
	if self.error then
		self:SetBackdropColor(0.7, 0.0, 0.0, 1)	
	else
		self:SetBackdropColor(0.5, 0.5, 0.5, 1)	
	end
end

function PlayerFrame:UpdateText()
	local s
	s=""
	if self.groupnum>0 and self.type~=PlayerFrame.TYPE_ROSTER_GROUP and self.type~=PlayerFrame.TYPE_ROSTER and self.class~="GROUP" then
		s=self.groupnum.." "..self.name
	else
		s=self.name
	end
	
	self.text:SetText(s)
	while self.text:GetStringWidth()>self:GetWidth()-10 and #s>5 do
		s=string.sub(s,1,#s-1)
		self.text:SetText(s)
	end

	
	if self.useCount>0 then 
		self.text:SetTextColor(0.5,0.5,0.5,1)
		return
	end
	if self.class=="GROUP" then
		self.text:SetTextColor(1.0,1.0,0.0,1)
	elseif self.class=="INFO" then
		self.text:SetTextColor(0.7,0.7,0.7,1)
	elseif self.class=="UNKNOWN" then
		self.text:SetTextColor(0.7,0.7,0.7,1)
	else
		self.text:SetTextColor(RAID_CLASS_COLORS[self.class].r,RAID_CLASS_COLORS[self.class].g,RAID_CLASS_COLORS[self.class].b,1)
	end
end

function PlayerFrame:ResetUseCount()
	if self.useCount>0 then
		self.useCount=0
		self:UpdateText()
	end
end

function PlayerFrame:IncreaseUseCount()
	self.useCount=self.useCount+1
	if self.useCount==1 then self:UpdateText() end
	if self.type==PlayerFrame.TYPE_ROSTER_GROUP then
		local i,f
		for i,f in ipairs(self.groupMembers) do
			f:IncreaseUseCount()
		end
	end
end
function PlayerFrame:DecreaseUseCount()
	self.useCount=self.useCount-1
	if self.useCount==0 then self:UpdateText() end
	if self.type==PlayerFrame.TYPE_ROSTER_GROUP then
		local i,f
		for i,f in ipairs(self.groupMembers) do
			f:DecreaseUseCount()
		end
	end
end

function PlayerFrame:EventClick(button, down)
	if self.type==PlayerFrame.TYPE_ROSTER then
		local was
		if not IsShiftKeyDown() then
			was=self.selected
			Core:ClearSelection()
			self.selected=was
		end
	
		if not self.selected then
			self:Select()
		else
			self:Deselect()
		end
	elseif self.type==PlayerFrame.TYPE_DROP_FULL then
		if IsShiftKeyDown() then
			self:DropTo(nil)
			Core.rosterFrame:UpdateUseCounts()
		end
	end
end

function PlayerFrame:Deselect(nocore)
	self.selected=false
	self:UnlockHighlight()
	if not nocore then Core:DeselectFrame(self) end
end

function PlayerFrame:Select(nocore)
	self.selected=true
	self:LockHighlight()
	if not nocore then Core:SelectFrame(self) end
end


function PlayerFrame:DropTo(f) -- f can be nil
	local swapped=false
	if f then
		if f.type==PlayerFrame.TYPE_DROP then
			f:Init(self.name,self.class,PlayerFrame.TYPE_DROP_FULL,self.groupnum)
		elseif f.type==PlayerFrame.TYPE_DROP_FULL then
			local name=self.name
			local class=self.class
			local groupnum=self.groupnum
			if self.type==PlayerFrame.TYPE_DROP_FULL then
				swapped=true
				self:Init(f.name,f.class,PlayerFrame.TYPE_DROP_FULL,f.groupnum)
			end
			f:Init(name,class,PlayerFrame.TYPE_DROP_FULL,groupnum)
		elseif f.type==PlayerFrame.TYPE_ROSTER then
			if Core.db.profile.rosterswaps then
				local id1=Core:FindPlayerRaidID(self.name)
				local id2=Core:FindPlayerRaidID(f.name)
				if id1 and id2 and id1~=id2 then
					if Core.simulate then
						Core:SimRaidSwap(id1,id2)
					else
						SwapRaidSubgroup(id1,id2)
					end
				end
			end
		elseif f.type==PlayerFrame.TYPE_ROSTER_GROUP then
			if Core.db.profile.rosterswaps then
				local id=Core:FindPlayerRaidID(self.name)
				if id then
					if Core.simulate then
						Core:SimRaidMove(id,f.groupnum)
					else
						SetRaidSubgroup(id,f.groupnum)
					end
				end
			end
		end
	end
	
	if not swapped and self.type==PlayerFrame.TYPE_DROP_FULL then
		self:Init(self.infoString,"INFO",PlayerFrame.TYPE_DROP,0)
	end
end



function PlayerFrame:EventDragStart()
	local candrag=false
	local i,f

	if self.type==PlayerFrame.TYPE_ROSTER or self.type==PlayerFrame.TYPE_ROSTER_GROUP or
				self.type==PlayerFrame.TYPE_DROP_FULL then
		candrag=true
	end
	
	
	if candrag then
		PlayerFrame.dragFrames={}
		if self.selected then
			selection=Core:GetSelectedFrames()
			for i,f in ipairs(selection) do
				newFrame=Core:GetPlayerFrame(f.name,f.class,PlayerFrame.TYPE_DRAGGING,f.groupnum)
				newFrame.origin=f
				PlayerFrame.dragFrames[#PlayerFrame.dragFrames+1]=newFrame
			end
		else
			Core:ClearSelection()
			newFrame=Core:GetPlayerFrame(self.name,self.class,PlayerFrame.TYPE_DRAGGING,self.groupnum)
			newFrame.origin=self
			PlayerFrame.dragFrames[#PlayerFrame.dragFrames+1]=newFrame
		end
		
		newFrame=PlayerFrame.dragFrames[1]
		newFrame:SetParent(Core.rosterSF) --self:GetParent())
		newFrame:ClearAllPoints()
		local x,y=GetCursorPosition()
		local s=UIParent:GetEffectiveScale()
		newFrame:SetPoint("CENTER",UIParent,"BOTTOMLEFT",x/s,y/s)
		for i=2,#PlayerFrame.dragFrames do
			newFrame=PlayerFrame.dragFrames[i]
			newFrame:SetParent(Core.rosterSF) --self:GetParent())
			newFrame:ClearAllPoints()
			newFrame:SetPoint("TOPLEFT",PlayerFrame.dragFrames[i-1],"BOTTOMLEFT",0,-2)
			newFrame:SetWidth(PlayerFrame.dragFrames[1]:GetWidth())
			newFrame:SetHeight(PlayerFrame.dragFrames[1]:GetHeight())
		end
		
		PlayerFrame.dragFrames[1]:StartMoving()
		PlayerFrame.dragging=true
	end
end

function PlayerFrame:EventDragStop()
	if PlayerFrame.dragFrames then
		local i,f
		
		PlayerFrame.dragFrames[1]:StopMovingOrSizing()
		
		local t,d,a=Core:FindMouseTarget()
		if not t and #PlayerFrame.dragFrames==1 then
			t=Core.rosterFrame:FindMouseTarget()
		end
		
		PlayerFrame.dragFrames[1].origin:DropTo(t)
		if #PlayerFrame.dragFrames>1 then
			
			targetnum=0
			targets=nil
			if t then
				targets=a:GetDropTargets()
				for i,f in ipairs(targets) do
					if f==t then 
						targetnum=i+1
						break
					end
				end
			end
			for i=2,#PlayerFrame.dragFrames do
				target=nil
				if targetnum>0 and targetnum<=#targets then
					target=targets[targetnum]
					targetnum=targetnum+1
				end
				PlayerFrame.dragFrames[i].origin:DropTo(target)
			end
		end
		
		for i,f in ipairs(PlayerFrame.dragFrames) do
			Core:ReleasePlayerFrame(f)
		end
		if t then Core:ClearSelection() end
		PlayerFrame.dragFrames=nil
		PlayerFrame.dragging=false
		if PlayerFrame.dragUpdatePending then
			PlayerFrame.dragUpdatePending=false
			Core:RaidChangedNow()
		end
		
		Core.rosterFrame:UpdateUseCounts()
	end
end

function PlayerFrame:EventMouseDown()
	PlayerFrame.dragging=true
end

function PlayerFrame:EventMouseUp()
	PlayerFrame.dragging=false
	if PlayerFrame.dragUpdatePending then
		PlayerFrame.dragUpdatePending=false
		Core:RaidChangedNow()
	end
end




