local ADDON_NAME="OrderOfBattle"
local _G = _G
local pairs = pairs
local ipairs = ipairs
local remove = table.remove
local type = type
local select = select
local next = next

local Core=OrderOfBattle
local PlayerFrame=OOBPlayerFrame

OOBRosterFrame = {Embed = Core.Embed}
local RosterFrame = OOBRosterFrame

local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME, true)

function RosterFrame:New(parent)
	if self ~= RosterFrame then return end
	local s=CreateFrame("FRAME",nil,parent,nil)
	RosterFrame:Embed(s)	
	
	s:ClearAllPoints()
	s:SetBackdrop({bgFile = nil, 
--					edgeFile = "Interface/Tooltips/UI-Tooltip-Border", edgeSize = 16,
					edgeFile = nil, edgeSize = 0,
					tile = true, tileSize = 16,  
					insets = { left = 0, right = 0, top = 0, bottom = 0 }})
	    
	s:SetWidth(100)
	
	s.groupFrames={}
	s.playerIndex={}
	
	s.raidLabel=Core:GetPlayerFrame(L["Raid"],"GROUP",PlayerFrame.TYPE_ROSTER_GROUP,0)
	s.raidLabel:ClearAllPoints()
	s.raidLabel:SetParent(s)
	s.raidLabel:SetPoint("TOP",s,"TOP",0,-12)	
	s.raidLabel:SetPoint("LEFT",s,"RIGHT",-88,0)
	s.raidLabel:SetPoint("RIGHT",s,"RIGHT",-12,0)
	s.raidLabel:Show()
	
	
	local prev,i
	for i=1,8 do
		s.groupFrames[i]=GroupFrame:New(s,i,prev)
		prev=s.groupFrames[i]
	end
	
	s:SetHeight(84*8+20)
	
	s:Show()
	
	return s
end

function RosterFrame:FindMouseTarget()
	local i,j,f,d
	local mx,my=GetCursorPosition()
	mx=mx/UIParent:GetEffectiveScale()
	my=my/UIParent:GetEffectiveScale()	
	local mind,minf=20,nil
	for i=1,8 do
		d=self.groupFrames[i].groupLabel:DistanceTo(mx,my)
		if d==0 then return self.groupFrames[i].groupLabel,0 end
		if d<mind then
			mind=d
			minf=self.groupFrames[i].groupLabel
		end
		
		for j,f in ipairs(self.groupFrames[i].players) do
			d=f:DistanceTo(mx,my)
			if d==0 then return f,0 end
			if d<mind then
				mind=d
				minf=f
			end
		end
	end
	return minf,mind
end

function RosterFrame:UpdateUseCounts()
	local i,j,p
	for i=1,8 do
		self.groupFrames[i].groupLabel:ResetUseCount()
		for j,p in ipairs(self.groupFrames[i].players) do
			p:SetError(false)
			p:ResetUseCount()
		end
	end

	local afs=Core:GetAllAssignmentFrames()
	local af,dts,dt,r
	for i,af in ipairs(afs) do
		dts=af:GetDropTargets()
		for j, dt in ipairs(dts) do
			dt:SetError(false)
			if dt.type==PlayerFrame.TYPE_DROP_FULL and dt.class~="GROUP" then
				r=self.playerIndex[dt.name]
				if r then
					if af:IsCounting() then
						r:IncreaseUseCount()
					end
					if r.groupnum>5 then
						dt:SetError(true,L["Player in group>5"])
						r:SetError(true,L["Player in group>5"])
					end
				else
					dt:SetError(true,L["Player not in raid"])
				end
			end
		end
	end
	
	for i=1,8 do
		for j,p in ipairs(self.groupFrames[i].players) do
			if self.groupFrames[i].players[j].useCount>1 then
				self.groupFrames[i].players[j]:SetError(true,L["Use count>1"])
				local n,m
				
				for n,af in ipairs(afs) do
					if af:IsCounting() then
						dts=af:GetDropTargets()
						for m, dt in ipairs(dts) do
							if dt.type==PlayerFrame.TYPE_DROP_FULL then
								if dt.name==self.groupFrames[i].players[j].name then
									dt:SetError(true,L["Use count>1"])
								end
							end
						end
					end
				end
			end
		end
	end
	
end

function RosterFrame:SelectPlayers(table)
	local i,j,p
	for i=1,8 do
		self.groupFrames[i].groupLabel:ResetUseCount()
		for j,p in ipairs(self.groupFrames[i].players) do
			if Core.tableFind(table,p.name) then
				p:Select()
			end
		end
	end	
end

function RosterFrame:UpdatePlayers()
	if PlayerFrame.dragging then
		PlayerFrame.dragUpdatePending=true
		return false
	end

	local i,name,class,group,online,isDead,frame,selection
	
	selection=Core:SelectionNames()
	Core.selectedFrames={}
	self:ClearPlayers()
	for i=1,40 do
		name,class,group,online,isDead = Core:GetRaidInfo(i)
		if not name then break end
		if group<=8 then 
			frame=Core:GetPlayerFrame(name,class,PlayerFrame.TYPE_ROSTER,group)
			self.groupFrames[group]:AddPlayer(frame)
			self.playerIndex[name]=frame
		end
	end
	for i=1,8 do
		self.groupFrames[i]:FinishUpdate()
	end
	self:SelectPlayers(selection)
	
	return true
end

function RosterFrame:ClearPlayers()
	local i,frame
	self.playerIndex={}
	self.playerIndex[L["Raid"]]=self.raidLabel
	for i,frame in ipairs(self.groupFrames) do
		frame:ClearPlayers()
		self.playerIndex[L["Group "]..i]=frame.groupLabel
	end
end

function RosterFrame:FindPlayer(name)
	return self.playerIndex[name]
end

GroupFrame = {Embed = Core.Embed}

function GroupFrame:New(parent,groupnum,prevFrame)
	if self ~= GroupFrame then return end
	local s=CreateFrame("FRAME",nil,parent,nil)
	GroupFrame:Embed(s)	
	
	s:ClearAllPoints()
	if prevFrame then
		s:SetPoint("TOP",prevFrame,"BOTTOM",0,-12)
		s:SetPoint("BOTTOM",prevFrame,"BOTTOM",0,-84)
	else		
		s:SetPoint("TOP",parent.raidLabel,"BOTTOM",0,-6)
		s:SetPoint("BOTTOM",parent.raidLabel,"BOTTOM",0,-78)		
	end
	s:SetPoint("LEFT",parent,"RIGHT",-90,0)
	s:SetPoint("RIGHT",parent,"RIGHT",-10,0)
	
	s:SetBackdrop({bgFile = nil, 
--					edgeFile = "Interface/Tooltips/UI-Tooltip-Border", edgeSize=8,
					edgeFile = nil, edgeSize=0,
					tile = true, tileSize = 16, 
					insets = { left = 0, right = 0, top = 0, bottom = 0 }})
	
	s.groupLabel=Core:GetPlayerFrame(L["Group "]..groupnum,"GROUP",PlayerFrame.TYPE_ROSTER_GROUP,groupnum)
	s.groupLabel:ClearAllPoints()
	s.groupLabel:SetParent(s)
	s.groupLabel:SetPoint("TOP",s,"TOP",0,-2)
	s.groupLabel:SetPoint("LEFT",s,"LEFT",2,0)
	s.groupLabel:SetPoint("RIGHT",s,"RIGHT",-2,0)
	s:Show()

	s.players={}

	return s
end

function GroupFrame:AddPlayer(playerFrame)
	self.players[#self.players+1]=playerFrame
end

function GroupFrame:FinishUpdate()
	table.sort(self.players,function(a,b)
		if a.name<b.name then return true else return false end
	end)
	local ref=self.groupLabel
	for i,f in ipairs(self.players) do
		f:SetParent(self)
		f:ClearAllPoints()
		f:SetPoint("TOP",ref,"BOTTOM",0,-2)
		f:SetPoint("LEFT",self,"LEFT",2,0)
		f:SetPoint("RIGHT",self,"RIGHT",-2,0)
		ref=f
		self.groupLabel.groupMembers[#self.groupLabel.groupMembers+1]=f
	end
end

function GroupFrame:ClearPlayers()
	local i,frame
	for i,frame in ipairs(self.players) do
		frame:ClearAllPoints()
		Core:ReleasePlayerFrame(frame)
	end
	self.groupLabel.groupMembers={}
	self.players={}
end
